'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = playerbaseStatistics;

var _simpleStatistics = require('simple-statistics');

var _simpleStatistics2 = _interopRequireDefault(_simpleStatistics);

var _roundTo = require('round-to');

var _roundTo2 = _interopRequireDefault(_roundTo);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var playtimeHoursGroups = [[1, 500], [500, 1000], [1000, 2000], [2000, 4000], [4000]];
var quantiles = [0.995, 0.99, 0.90, 0.80, 0.50, 0.25, 0.10, 0.01];

function playerbaseStatistics(playerbase) {
  var statistics = {};
  var graphData = {};

  // Add a statistic & graph for a partial playerbase
  function pushPlayerbaseData(key, partialPlayerbaseValues) {
    statistics[key] = calculateStatistics(partialPlayerbaseValues);
    graphData[key] = calculateGraphData(partialPlayerbaseValues);
  }

  // Always add a key for the whole playerbase
  pushPlayerbaseData('all', playerbase.map(function (x) {
    return x.value;
  }));

  // Add a key each per playtime group
  playtimeHoursGroups.map(function (group) {
    var key = 'playtime' + (group.length === 1 ? group : group.join('to'));

    // Filter the entries that match the playtime group
    var partialPlayerbaseValues = playerbase.filter(function (x) {
      if (x.playtime < group[0] * 60 * 60) {
        return false;
      }

      if (group[1] && x.playtime > group[1] * 60 * 60) {
        return false;
      }

      return true;
    }).map(function (x) {
      return x.value;
    });

    // Calculate and push the data
    pushPlayerbaseData(key, partialPlayerbaseValues);
  });

  // All calculations done, let's return!
  return {
    leaderboard: playerbase.reverse().slice(0, 25),
    statistics: statistics,
    graphData: graphData
  };
}

function calculateStatistics(playerbase) {
  if (playerbase.length === 0) return {};

  // Get the basic statistics
  var statistics = {
    min: _simpleStatistics2.default.minSorted(playerbase),
    max: _simpleStatistics2.default.maxSorted(playerbase),
    mean: (0, _roundTo2.default)(_simpleStatistics2.default.mean(playerbase), 2),
    median: (0, _roundTo2.default)(_simpleStatistics2.default.medianSorted(playerbase), 2)
  };

  // Get the value for each quantile
  quantiles.map(function (quantile) {
    var key = 'quantile' + (quantile * 100).toString().replace('.', ',');
    statistics[key] = (0, _roundTo2.default)(_simpleStatistics2.default.quantileSorted(playerbase, quantile), 2);
  });

  return statistics;
}

function calculateGraphData(playerbase) {
  if (playerbase.length === 0) return [];

  // Generate 100 points for the graph, using the quantiles
  var points = [];
  for (var i = 0; i < 100; i++) {
    points.push((0, _roundTo2.default)(_simpleStatistics2.default.quantileSorted(playerbase, i / 100), 2));
  }

  return points;
}